<?php
/**
 * @version    1.0.2
 * @package    com_gadecisions
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gadecisions\Administrator\Helper;

// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\User\UserHelper;
use \Joomla\CMS\User\UserFactoryInterface;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\Filesystem\Path;
use \Joomla\Filesystem\File;
use \Joomla\Filesystem\Folder;
use \Joomla\CMS\User\User;
use \Joomla\CMS\Date\Date;
use \Joomla\CMS\Mail\MailTemplate;
use \Joomla\CMS\Access\Access;
use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Installer\Installer;
use \Joomla\Session\SessionInterface;
use \Joomla\CMS\Application\SiteApplication;
use \GlennArkell\Component\Gadecisions\Administrator\Helper\GaemailHelper;

/**
 * Main component helper.
 */
class GadecisionsHelper
{
    /** --------------------------------------------------------------------------------------------------   **/
    /** -----------------------------  Regularly used for most components       --------------------------   **/
    /** --------------------------------------------------------------------------------------------------   **/

	/**
	 * Build the HTTP query array
	 * @param array of the http query if the http query already prepared and we want to add more
	 * @param string view or a task
	 * @param string control method to be used
	 * @param string reference indicator
	 * @param string/int reference string or id
	 * @return array
	 * @ hint - this is used with http_build_query($query_string, '', '&amp;') to build a clean url
	 */
	public static function getHTTPQuery($existQ = null, $viewTask = 'view', $contModel = 'motion', $ref = 'id', $linkId = 0)
	{
        if ($contModel == 'article') {
            $compName = 'com_content';
        } else {
            $compName = 'com_gadecisions';
        }
        if (!$existQ) {
			$query_string = array();
			$query_string['option'] = $compName;
			$query_string[$viewTask] = $contModel;
			if ($ref) {
				$query_string[$ref] = $linkId;
			}
		} else {
			$query_string = $existQ;
			$query_string[$ref] = $linkId;
		}

		return $query_string;
	}

    /**
     * Gets todays date object based on global timezone settings
     */
    public static function getTodaysDate()
	{
		//$tz = Factory::getConfig()->get('offset');
		//$date = Factory::getDate('now', $tz);
		//$today = date_format($date,'Y-m-d H:i:s');
		$today = Factory::getDate()->toSql();

		return $today;
	}

    /**
     * Prints out a variable value in human readable format
     */
    public static function gaprint($val){
        echo '<pre>Test<br />';
        \print_r($val);
        echo  '</pre>';
    }

    /**
     * Gets the user record for the specific id reference
     */
    public static function getSpecificUser($id = 0)
	{
		if ($id) {
			$container = Factory::getContainer();
			$userFactory = $container->get(UserFactoryInterface::class);
			$user = $userFactory->loadUserById($id);
		} else {
			$user = Factory::getApplication()->getIdentity();
		}
		unset($user->password);
		unset($user->activation);
		unset($user->params);
		unset($user->lastResetTime);
		unset($user->resetCount);
		unset($user->otpKey);
		unset($user->otep);
		unset($user->requireReset);

		return $user;
	}

	/**
	 * Set up a web asset object for use
	 */
	public static function setupGAWebApp()
	{
        // Boot the DI container.
        $container = Factory::getContainer();
        // Alias the session service key to the web session service.
        $container->alias(SessionInterface::class, 'session.web.site');
        // Get the application.
        $app      = $container->get(SiteApplication::class);
        //$template = $app->getTemplate(true);
        //$params   = $template->params;
        //$wa = Factory::getApplication()->getDocument()->getWebAssetManager();
        
        return $app;
	}

    /**
     * Load template over-rides when using modal view
     * This is important for Cloud White template scheme
     * because modals have the primary css over-ride any
     * css loaded in the template index.php file
     */
    public static function loadTmplStyleModal($wa)
	{
        $tmpl = Factory::getApplication()->getTemplate(true);

//         if ($tmpl->template != 'cassiopeia') {
//             // load cassiopeia template css - for modals only
//             $tmplCSS = JPATH_BASE.'/media/templates/site/cassiopeia/css/template.css';
//             if (\file_exists($tmplCSS)) {
//                 $wa->useStyle('cassiopeia');
//             }
//         }

        // if using rkicNNsite then check for cloudwhite
        if ($tmpl->params->get('colorName') == 'colors_white') {
        	$wa->addInlineStyle('.btn-primary {background-color:'.$tmpl->params->get('btnPbgColor').' !important;}');
        	$wa->addInlineStyle('.nav-link {color:var(--template-contrast) !important;}');
        	$wa->addInlineStyle('.form-check-input:checked, .form-select[multiple] option:checked, [multiple].custom-select option:checked {background-color:'.$tmpl->params->get('btnPbgColor').' !important;}');
        	$wa->addInlineStyle('.page-item.active .page-link {color: var(--rkic41site-color-link); background-color: #c1cee1 !important; border-color: #dfe3e7 !important;}');
        	$wa->addInlineStyle('.form-select[multiple] option:checked, [multiple].custom-select option:checked {background-color: var(--rkic41site-color-primary-border) !important;}');
        }

		return true;
	}

	/**
	 * Get the version of the component
	 * @return version element of manifest
	 */
	public static function getComponentVersion()
	{
		$componentXML = Installer::parseXMLInstallFile(Path::clean(JPATH_ADMINISTRATOR . '/components/com_gadecisions/gadecisions.xml'));

		return $componentXML['version'];
	}

    /**
     * Gets the edit permission for an user
     * @param   mixed  $item  The item
     * @return  bool
     */
    public static function canUserEdit($user, $item)
    {
        $permission = false;

        if ($user->authorise('core.edit', 'com_gadecisions')) {
            $permission = true;
        } else {
            if (isset($item->created_by) && $item->created_by) {
                if ($user->authorise('core.edit.own', 'com_gadecisions') && $item->created_by == $user->id) {
                    $permission = true;
                }
            } else {
                if ($user->authorise('core.create', 'com_gadecisions')) {
                    $permission = true;
                }
            }
        }

        return $permission;
    }

	/**
	 * Get the Model from another component for use
	 * @param   string  $name    The model name.
	 * @param   string  $prefix  The class prefix.
	 * @param   array   $config  Configuration array for model.
	 * @return object	The model
	 */
	public function getForeignModel($name = 'Actionlog', $prefix = 'Administrator', $config = array('ignore_request' => true))
	{
		$fmodel = Factory::getApplication()->bootComponent('com_actionlogs')->getMVCFactory()->createModel($name, $prefix, $config);

		return $fmodel;
	}

	/**
	 * Create a new record based on params
	 * @param   string  $table  Table name.
	 * @param   string  $key    The reference key
	 * @param   string  $value  Key value tyo be loaded
	 * @return boolean	false on failure
	 */
	public static function createNewRecord($table, $key, $value, $params)
	{
        $state = $params->get('auto_create', 0) ? 1 : 0;
        $new_rec = new \stdClass();
        $new_rec->id = 0;
        $new_rec->state = $state;
        $new_rec->motion_id = $key;
        $new_rec->user_id = $value;
        $new_rec->comment = 'Auto created.';

		try {
			$result = Factory::getContainer()->get('DatabaseDriver')->insertObject($table, $new_rec);
			Factory::getApplication()->enqueueMessage(Text::_('COM_GADECISIONS_NEWREC_SUCCESSFUL'), 'notice');
		} catch (RuntimeException $e) {
			Factory::getApplication()->enqueueMessage($e->getMessage().' '.Text::_('COM_GADECISIONS_NEWREC_FAILED'), 'danger');
			return false;
		}
		return true;
	}

    /** --------------------------------------------------------------------------------------------------   **/
    /** -----------------------------  Get options for selection fields etc     --------------------------   **/
    /** --------------------------------------------------------------------------------------------------   **/

	public static function getListOptions($table = '#__users', $label = 'Member', $fld_value = 'id', $fld_text = 'name' )
	{
        // setup for other languages regarding the selection
        $sel_label = strtoupper('COM_GADECISIONS_SELECT_'.$label);
        $sel_label = Text::_($sel_label);
		// get the records
        $db = Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - Select '.$label.' - " as "text" UNION SELECT '.$db->quoteName($fld_value).' as value, '.$db->quoteName($fld_text).' as text ');
		$query->from( $db->quoteName($table) );
		if ($table == '#__users') {
            $query->where(' block = 0' );
        } else {
            $query->where(' state = 1' );
        }
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

	public static function getCustomFieldsOptions($ext = 'com_gadecisions.motionform', $label = 'Field', $field = 'title' )
	{
		// get the categories
        $db = Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - Select '.$label.' - " as "text" UNION SELECT id as value, '.$field.' as text ');
		$query->from( ' #__fields ' );
		$query->where(' state = 1' );
		$query->where(' context = '.$db->quote($ext) );
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

	public static function getCategoryOptions($ext = 'com_gadecisions', $label = 'Category Type', $field = 'title' )
	{
		// get the categories
        $db = Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - Select '.$label.' - " as "text" UNION SELECT id as value, '.$field.' as text ');
		$query->from( ' #__categories ' );
		$query->where(' published = 1' );
		$query->where(' extension = '.$db->quote($ext) );
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

	public static function getMotionOptions()
	{
		// get the records
        $db = Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - Select Motion - " as "text" UNION SELECT '.$db->quoteName('id').' as value, CONCAT('.$db->quoteName('id').'," - ",DATE_FORMAT('.$db->quoteName('motion_date').', "%Y-%m-%d")) as text ');
		$query->from( $db->quoteName('#__gadecisions_motions') );
        $query->where(' state = 1' );
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

    /**
    *   Method to get the required record
    *   @param string $table table name
    *   @param int $id id reference
    *   @return object record data
    */
	public static function getMotion($id)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.*, m.name AS mov_id_name, s.name AS sec_id_name, g.title AS group_id_name ')
    		->from($db->quoteName('#__gadecisions_motions', 'a'))
    		->join('LEFT',$db->quoteName('#__users', 'm').' ON m.id = a.mov_id' )
    		->join('LEFT',$db->quoteName('#__users', 's').' ON s.id = a.sec_id' )
    		->join('LEFT',$db->quoteName('#__usergroups', 'g').' ON g.id = a.group_id' )
    		->where(' a.id = '.(int) $id );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObject();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' Record');
	        return false;
	    }

	}

    /** --------------------------------------------------------------------------------------------------   **/
    /** -----------------------------  Get some core reference type data elements   ----------------------   **/
    /** --------------------------------------------------------------------------------------------------   **/

    /**
    *   Method to get values for a member based on field id
    *   @param int $id field_id reference
    *   @param int $userId user id reference
    *   @return object field name and value
    */
	public static function getCustomFieldValue($id = 0, $userId = 0)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' b.title As fieldName, a.value AS fieldValue ');
		$query->from(' #__fields_values AS a ');
		$query->join('LEFT', ' #__fields AS b ON b.id = a.field_id');
		$query->where(' a.field_id = '.(int) $id );
		$query->where(' a.item_id = '.(int) $userId );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObject();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' Field Values');
	        return false;
	    }

	}

    /**
    *   Method to get the profile fields available
    *   @return array of object records
    */
	public static function getProfileFieldsOptions()
	{
        $params = ComponentHelper::getParams('com_gadecisions');
        $proflocal = $params->get('profile_suffix');
        $prof_key = 'profile'.$proflocal.'.';
        $keyLen = strlen($prof_key);
		$startkey = $keyLen+1;
		$sel_field = Text::_('COM_GADECISIONS_SELECT_FIELD');

		// get the user records in the supplier group
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - '.$sel_field.' - " as "text" UNION SELECT profile_key as value, substr(profile_key,'.$startkey.') as text ');
		$query->from( $db->quoteName('#__user_profiles') );
		$query->where(' substr(profile_key,1,'.$keyLen.') = '.$db->Quote($prof_key) );
		$query->group(' profile_key ' );
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObjectList();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

	}

    /** --------------------------------------------------------------------------------------------------   **/
    /** -----------------  Get basic Record/s providing a table name and other reference data ------------   **/
    /** --------------------------------------------------------------------------------------------------   **/

    /**
    *   Method to get the required record
    *   @param string $table table name
    *   @param int $id id reference
    *   @return object record data
    */
	public static function getRecord($table = '', $id = 0)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.* ');
		$query->from($db->quoteName($table, 'a'));
		$query->where(' a.id = '.(int) $id );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObject();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' Record');
	        return false;
	    }

	}

    /**
    *   Method to get the required record
    *   @param string $table table name
    *   @param string $field field name to match on the id reference
    *   @param int $id id reference
    *   @return object record data
    */
	public static function getRecordList($table = '', $field = '', $id = 0)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.* ');
		$query->from($db->quoteName($table, 'a'));
		$query->where($db->quoteName($field).' = '.(int) $id );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' RecordList');
	        return false;
	    }

	}

    /**
    *   Method to get the required record
    *   @param object $user to check
    *   @param int $id motion id reference
    *   @return boolean
    */
	public static function hasVoted($userId, $id, $decider)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.* ')
    		->from($db->quoteName('#__gadecisions_voters', 'a'))
    		->where($db->quoteName('motion_id').' = '.(int) $id )
    		->where($db->quoteName('state').' = 1');
    	if ($decider) {
            $query->where($db->quoteName('decider').' = '.(int) $userId );
        } else {
            $query->where($db->quoteName('user_id').' = '.(int) $userId );
	    }
		$db->setQuery((string)$query);

	    try {
	        return !empty($db->loadObject())? true : false;
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' RecordList');
	        return false;
	    }

	}

    /**
    *   Method to get the required records
    *   @param int $id motion id reference
    *   @return array of objects
    */
	public static function getVoters($id)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.*, u.name ')
    		->from($db->quoteName('#__gadecisions_voters', 'a'))
    		->join('LEFT', $db->quoteName('#__users', 'u') . ' ON u.id = a.user_id ')
    		->where($db->quoteName('motion_id').' = '.(int) $id )
    		->where($db->quoteName('state').' = 1');
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' RecordList');
	        return false;
	    }

	}

    /**
    *   Method to get the required records
    *   @param int $id motion id reference
    *   @return array of objects
    */
	public static function resetVoters($id)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->update($db->quoteName('#__gadecisions_voters'))
    		->set($db->quoteName('state').' = -2 ' )
    		->where($db->quoteName('motion_id').' = '.(int) $id )
    		->where($db->quoteName('state').' = 1');
		$db->setQuery((string)$query);

	    try {
	        return $db->execute();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' RecordList');
	        return false;
	    }

	}

    /**
    *   Method to get the number of votes for a specific motion/field/value
    *   @param int $id motion id reference
    *   @return array of objects
    */
	public static function countCFVotes($motion, $field, $value)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' count(*) ')
    		->from($db->quoteName('#__gadecisions_cfvotes'))
    		->where($db->quoteName('motion_id').' = '.(int) $motion )
    		->where($db->quoteName('field_id').' = '.(int) $field )
    		->where($db->quoteName('vote').' = '.$db->quote($value) )
    		->where($db->quoteName('state').' = 1');
		$db->setQuery((string)$query);

	    try {
	        return $db->loadResult();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' Custom Field Vote Count');
	        return false;
	    }

	}

    /** --------------------------------------------------------------------------------------------------   **/
    /** ---------------------------  eMail preparation using the MailTemplate system  --------------------   **/
    /** --------------------------------------------------------------------------------------------------   **/

	/**
	* Notify the user/client/member etc through email
	* @param   array   $data passed by booking screen
	* @param   string  $tmpl  mandatory (template ext)
	* @param   integer $id booking reference if existing
	* @return true
	*/
	public static function notifyUser($data, $tmpl)
	{
		if ($data['id']) {
    		$params  = ComponentHelper::getParams('com_gadecisions');
    		
    		$item = GadecisionsHelper::getRecord('#__gadecisions_motions', $data['id']);

    		// setup the data to include in email
    		$data['motion'] = $item->motion;
    		$data['motion_date'] = HtmlHelper::date($item->motion_date, Text::_('COM_GADECISIONS_DISPLAY_DATE'));

            $data['recips'] = array(array('email'=>$item->user_email, 'name'=>$item->user_name));
    		$data['cc_recips'] = array(array('email'=>$item->user_email, 'name'=>$item->user_name));

			$viewLink = GadecisionsHelper::getHTTPQuery(null, 'view', 'motion', 'id', $item->id);
			$link = 'index.php?'.http_build_query($viewLink, '', '&amp;');

            if ($params->get('tmpl_email', 0)) {
                GaemailHelper::sendEmailTemplate('com_gadecisions.'.$tmpl, $data, null, $link, null);
            }
		} else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_GADECISIONS_NO_ID_MESSAGE'), 'warning');
        }

		return true;
	}

    /** --------------------------------------------------------------------------------------------------   **/
    /** -----------------------  Some specific processes for this particular component  ------------------   **/
    /** --------------------------------------------------------------------------------------------------   **/

	/**
	* Check vote counts
	* @param   integer $id motion
	* @return true
	*/
	public static function checkVotingNumbers($id)
	{
		if ($id) {
    		$params  = ComponentHelper::getParams('com_gadecisions');
    		$sc_group = $params->get('sc_user_gp', 0);

    		$item = self::getRecord('#__gadecisions_motions', $id);
            $voters = self::getRecordList('#__gadecisions_voters', 'a.motion_id', $id);
            $votersRecorded = empty($voters) ? 0 : \count($voters);

            // check for custom field settings
            if ($item->field_id) {
                // get cf votes
                $votes = self::getRecordList('#__gadecisions_cfvotes', 'a.motion_id', $id);
                $votesRecorded = \count($votes);
            } else {
                $votesRecorded = $item->votes_for + $item->votes_against;
    		}

            if ($votesRecorded == $votersRecorded) {
                return true;
            }
            return false;
		} else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_GADECISIONS_NO_ID_MESSAGE'), 'warning');
        }

		return true;
	}

	/**
	* Check vote counts
	* @param   integer $id motion
	* @return true
	*/
	public static function notifyScrutineers($id)
	{
		if ($id) {
    		$params  = ComponentHelper::getParams('com_gadecisions');
    		$sc_group = $params->get('sc_user_gp', 0);

    		$item = self::getRecord('#__gadecisions_motions', $id);
            $scrutineers = Access::getUsersByGroup($sc_group, false);
            foreach ($scrutineers as $s) {
                $recips[] = array('email'=>$s->email, 'name'=>$s->name);
            }

    		// setup the data to include in email
    		$data['motion'] = $item->title;
    		$data['motion_date'] = HtmlHelper::date($item->motion_date, Text::_('COM_GADECISIONS_DISPLAY_DATE'));

            $data['recips'] = $recips;
            $data['votes'] = $votesRecorded;
            $data['voters'] = $votersRecorded;

			$viewLink = self::getHTTPQuery(null, 'view', 'motion', 'id', $item->id);
			$link = 'index.php?'.http_build_query($viewLink, '', '&amp;');

            if ($params->get('tmpl_email', 0)) {
                GaemailHelper::sendEmailTemplate('com_gadecisions.scrutineer', $data, null, $link, null);
            }
            return false;
		} else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_GADECISIONS_NO_ID_MESSAGE'), 'warning');
        }

		return true;
	}

}

