<?php

/**
 * @package    Com_Gadecisions
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\Filesystem\File;
use \Joomla\Filesystem\Folder;
use \Joomla\Filesystem\Path;
use \Joomla\CMS\Table\Table;
use \Joomla\CMS\Mail\MailTemplate;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\CMS\Installer\Installer;
use \Joomla\CMS\Installer\InstallerScript;
use \Joomla\CMS\Filter\OutputFilter;
// use \Joomla\CMS\Installer\Adapter\InstallerAdapter;
// use \Joomla\CMS\Installer\Adapter\ComponentAdapter;
// use \Joomla\CMS\Installer\Adapter\ModuleAdapter;
// use \Joomla\CMS\Installer\Adapter\PluginAdapter;

/**
 * Updates the structure of the component
 */
class com_gadecisionsInstallerScript extends InstallerScript
{
	/**
	 * The title of the component (printed on installation and uninstallation messages)
	 * @var string
	 */
	protected $extension = 'Decisions System';

	private $app;

	public $compName = 'gadecisions';

	public $mailTmplSuffixs = array("motions", "scrutineer");

	public $mailTags = array("sitename","link_text","motion","motion_date", "votes", "voters");

	public $version = '1.0.2';

	public $mainView = 'motions';


	/**
	 * The minimum Joomla! version required to install this extension
	 * @var   string
	 */
	protected $minimumJoomla = '4.0';

	/**
	 *  Constructor
	 */
	public function __construct()
	{
		$this->app = Factory::getApplication();

        $this->gTours = array(
           'COM_GADECISIONS_GUIDEDTOUR_CONFIG_LBL'=>'COM_GADECISIONS_GUIDEDTOUR_CONFIG_DESC',
           'COM_GADECISIONS_GUIDEDTOUR_MOTION_LBL'=>'COM_GADECISIONS_GUIDEDTOUR_MOTION_DESC'
           );
	}

	/**
	 * Method called before install/update the component. Note: This method won't be called during uninstall process.
	 * @param   string $type   Type of process [install | update]
	 * @param   mixed  $parent Object who called this method
	 * @return boolean True if the process should continue, false otherwise
     * @throws Exception
	 */
	public function preflight($type, $parent)
	{
		// $parent is the class calling this method
		echo '<p>' . Text::_('COM_'.STRTOUPPER($this->compName).'_PREFLIGHT_'.STRTOUPPER($type).'_TEXT') . '</p>';

		if (JVERSION < $this->minimumJoomla) {
			Factory::getApplication()->enqueueMessage(Text::sprintf('GA_INSTALL_CHECK_FAIL',$this->minimumJoomla,JVERSION), 'danger');
			return false;
		} else {
			Factory::getApplication()->enqueueMessage(Text::sprintf('GA_INSTALL_CHECK_OK',$this->minimumJoomla,JVERSION), 'message');
			return parent::preflight($type, $parent);
		}

	}

	/**
	 * Method to install the component
	 * @param   mixed $parent Object who called this method.
	 * @return void
	 * @since 0.2b
	 */
	public function install($parent)
	{
		// $parent is the class calling this method
		echo '<p>' . Text::_('COM_'.STRTOUPPER($this->compName).'_INSTALL_TEXT') . '</p>';

		$this->addDashboardMenu($this->compName, $this->compName);

		$categories = array(
				"uncategorised"=>""
				);
		$this->createCategories('com_'.$this->compName, $categories);

		//$this->installPlugins($parent);

		// install the Mail Templates
		$this->installMailTemplates();
	}

	/**
	 * Method to update the component
	 * @param   mixed $parent Object who called this method.
	 * @return void
	 */
	public function update($parent)
	{
		// $parent is the class calling this method
		echo '<p>' . Text::_('COM_'.STRTOUPPER($this->compName).'_UPDATE_TEXT') . '</p>';

		$dashB = $this->checkDashboard($this->compName);
		if (!$dashB) {
			$this->addDashboardMenu($this->compName, $this->compName);
		}

		//$this->uninstallPlugins($parent);

		// install the Mail Templates
		$this->installMailTemplates();

	}

	/**
	 * Method to uninstall the component
	 * @param   mixed $parent Object who called this method.
	 * @return void
	 */
	public function uninstall($parent)
	{
		// $parent is the class calling this method
		echo '<p>' . Text::_('COM_'.STRTOUPPER($this->compName).'_UNINSTALL_TEXT') . '</p>';

		$dashB = $this->checkDashboard($this->compName);
		if ($dashB) {
			foreach ($dashB as $dash) {
                $this->removeDashboardMenu($dash->id);
            }
		}

        foreach ($this->mailTmplSuffixs as $tmpl) {
            $template_id = 'com_'.$this->compName.'.'.$tmpl;
            $tmplExists = $this->checkMailTemplates($template_id);
            if ($tmplExists) {
                $this->removeMailTemplate($template_id);
            }
        }

		/* --------------------------------  Guided Tours  ------------------------------ */
        // if guided tours exist, remove them
        if (isset($this->gTours) && is_array($this->gTours) && !empty($this->gTours)) {
            foreach ($this->gTours as $gtUid => $title) {
                $tourUID =  STRTOLOWER($this->compName.'-'.$gtUid);
                $tourExists = $this->checkTourExists($tourUID);
                if ($tourExists) {
                    $this->removeTour($tourExists->id);
                }
            }
        }

	}

	/**
	 * @param   string $type   type
	 * @param   string $parent parent
	 * @return boolean
	 */
	public function postflight($type, $parent)
	{
		// $parent is the class calling this method
		echo '<p>' . Text::_('COM_'.STRTOUPPER($this->compName).'_POSTFLIGHT_'.STRTOUPPER($type).'_TEXT') . '</p>';

		if (STRTOUPPER($type) == 'INSTALL') {
			// do something

            // check if Guided Tours is ok for install
            if (JVERSION <= '4.4.3') {
                // don't install guided tours
                Factory::getApplication()->enqueueMessage(Text::sprintf('GA_INSTALL_NOGT',JVERSION), 'message');
            } else {
                // reinstall parameter to be passed
                $this->checkGuidedTours(false);
            }

		}

		if (STRTOUPPER($type) == 'UPDATE') {
			// do something

    		// clean up old sql update file
			$path = Path::clean( JPATH_ADMINISTRATOR . '/components/' . $this->compName . '/src/Field/');
			$this->deleteFiles($path, 'CustfldmotionsField', 'php');

            // check if Guided Tours is ok for install
            if (JVERSION <= '4.4.3') {
                // don't install guided tours
                Factory::getApplication()->enqueueMessage(Text::sprintf('GA_INSTALL_NOGT',JVERSION), 'message');
            } else {
                // reinstall parameter to be passed
                $this->checkGuidedTours(false);
            }

		}

		return true;
	}

	/**
	 * @param   string  $parent  parent
	 * @return void
	 */
	public function discover_install($parent)
	{
		return self::install($parent);
	}

	/**
	 * *********************  Regularly used installation stuff  *******************************
	 */

	/**
	 * Create some special Category records
	 * @param   none
	 * @return boolean
	 */
	public function createCategories($extension, $cattype)
	{
        $app = Factory::getApplication();
		
		foreach ($cattype as $title => $desc) {
            $category = Table::getInstance('Category');
            $category->extension = $extension;
            $category->title = $title;
            $category->alias = OutputFilter::stringUrlSafe($title);
            $category->description = $desc;
            $category->published = 1;
            $category->access = 1;
            $category->params = '{"category_layout":"","image":"","image_alt":""}';
            $category->metadata = '{"page_title":"","author":"","robots":""}';
            $category->language = '*';
            // Set the location in the tree
            $category->setLocation(1, 'last-child');
            // Check to make sure our data is valid
            if (!$category->check()) {
                $app->enqueueMessage(Text::_($category->getError()));
                return false;
            }
            // Now store the category
            if (!$category->store(true)) {
                $app->enqueueMessage(Text::_($category->getError()));
                return false;
            }
	 	}
        // Build the path for our category
        $category->rebuildPath($category->id);
        echo '<p>' . Text::_('New categories created') . '</p>';

		return true;
	}

	/**
	 * *********************  Guided Tours Setup  *******************************
	 */

	/**
	 * Check for Guided Tours
	 * @param   boolean  true if reinstall is to be performed
	 */
	public function checkGuidedTours($reinstall)
	{
        // check for guided tours and create if required
        if (isset($this->gTours) && is_array($this->gTours) && !empty($this->gTours)) {
            foreach ($this->gTours as $gtUid => $desc) {
                $tourUID =  STRTOLOWER($this->compName.'-'.Text::_($gtUid));
                $tourExists = $this->checkTourExists($tourUID);

                if ($tourExists && $reinstall) {
                    // remove old GTs because changes made and we need to remove old steps
                    $this->removeTour($tourExists->id);
                    $tourExists = false;
                }

                if (!$tourExists) {
                    // set the start url and create tour
                    if ($gtUid == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_LBL') {
                        $url = 'administrator/index.php?option=com_'.$this->compName.'&view='.$this->mainView;
                    } elseif ($gtUid == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_LBL') {
                        $url = 'administrator/index.php?option=com_cpanel&view=cpanel&dashboard='.$this->compName;
                    } else {
                        continue;
                    }
                    $gtID = $this->createGuidedTour($gtUid, $desc, $url);
                    $this->app->enqueueMessage(Text::sprintf('COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_LOADED', Text::_($gtUid), $gtID), 'notice');
                    // create steps array
                    $recs = $this->setupGuidedTourSteps($gtUid);
                    // create tour steps
                    $this->createGuidedTourSteps($gtID, $recs);
                }
            }
        }

	}

	/**
	 * Check if a Guided Tour entry exists
	 * @param   string  $tourUid for the uid reference
	 * @return boolean or object
	 */
	public function checkTourExists($tourUid)
	{
        $result = false;
		$db = Factory::getContainer()->get('DatabaseDriver');
	    $db->setQuery(' SELECT * FROM #__guidedtours WHERE extensions = '.$db->Quote('["com_'.$this->compName.'"]') . ' AND uid = '.$db->Quote($tourUid) );
		try {
		    $result = $db->loadObject();
		} catch (RuntimeException $e) {
		    $this->app->enqueueMessage($e->getMessage(), 'danger');
		}

		return $result;
	}

	/**
	 * Create a Guided Tour record
	 * @param   string  $tourName for the uid reference and title
	 * @param   string  $url to set where the tour starts from
	 * @return boolean or record id
	 */
	public function createGuidedTour($tourName, $desc, $url)
	{
        $today = Factory::getDate()->toSql();
        $userId = Factory::getApplication()->getIdentity()->id;
        $db = Factory::getContainer()->get('DatabaseDriver');
        $gtour = new \stdClass();
        $gtour->title = $tourName;
        $gtour->uid = $this->compName.'-'.STRTOLOWER(Text::_($tourName));
        $gtour->description = $desc;
        $gtour->extensions = '["com_'.$this->compName.'"]';
        $gtour->url = $url;
        $gtour->created = $today;
        $gtour->created_by = $userId;
        $gtour->modified = $today;
        $gtour->modified_by = $userId;
        $gtour->language = '*';
        $gtour->published = 1;
        $gtour->note = '';
        $gtour->access = 1;
		try {
		    $result = $db->insertObject('#__guidedtours', $gtour);
            return $db->insertid();
		} catch (RuntimeException $e) {
		    $this->app->enqueueMessage($e->getMessage(), 'warning');
		    return false;
		}

	}

	/**
	 * Create a Guided Tour record
	 */
	public function createGuidedTourSteps($id, $recs)
	{
        $today = Factory::getDate()->toSQL();
        $userId = Factory::getApplication()->getIdentity()->id;
        foreach ($recs as $data) {
            $db = Factory::getContainer()->get('DatabaseDriver');
            $gtstep = new \stdClass();
            $gtstep->tour_id = $id;
            $gtstep->title = $data['title'];
            $gtstep->published = 1;
            $gtstep->description = $data['desc'];
            $gtstep->position = $data['position'];
            $gtstep->target = $data['target'];
            $gtstep->type = $data['type'];
            $gtstep->interactive_type = $data['intertype'];
            $gtstep->url = $data['url'];
            $gtstep->language = '*';
            $gtstep->created = $today;
            $gtstep->created_by = $userId;
            $gtstep->modified = $today;
            $gtstep->modified_by = $userId;
    		try {
    		    $result = $db->insertObject('#__guidedtour_steps', $gtstep);
    		} catch (RuntimeException $e) {
    		    $this->app->enqueueMessage($e->getMessage(), 'warning');
    		}
		}
        return true;

	}

	/**
	 * Removes the guided tour entries
	 * @param int $id The guided tour ID reference
	 * @return  void
	 */
	public function removeTour($id)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');
	    $db->setQuery(' DELETE FROM #__guidedtours WHERE id = '. (int) $id );
		try {
		    $db->execute();
			$this->app->enqueueMessage(Text::_('COM_'.STRTOUPPER($this->compName).'_REMOVE_GUIDEDTOUR_SUCCESS'), 'notice');
		} catch (RuntimeException $e) {
		    $this->app->enqueueMessage($e->getMessage(), 'danger');
		}

		$db1 = Factory::getContainer()->get('DatabaseDriver');
	    $db1->setQuery(' DELETE FROM #__guidedtour_steps WHERE tour_id = '. (int) $id );
		try {
		    $db1->execute();
			$this->app->enqueueMessage(Text::_('COM_'.STRTOUPPER($this->compName).'_REMOVE_GUIDEDTOURSTEPS_SUCCESS'), 'notice');
		} catch (RuntimeException $e1) {
		    $this->app->enqueueMessage($e1->getMessage(), 'danger');
		}
	}

	/**
	 * Set up the step entries
	 * @param   integer  $ref to identify which tour the steps belong to
	 * types - 0 = Next, 1 = Redirect, 2 = Interactive
	 * interactive types - 1 = Form Submit, 2 = Text Field, 4 = Button, 3 = Other
	 * @return boolean or array
	 */
	public function setupGuidedTourSteps($ref)
	{
        $recs = array();

        if ($ref == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_LBL') {
            $returnURL = 'administrator/index.php?option=com_'.$this->compName.'&view='.$this->mainView;
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_NEW_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_NEW_DESC',
                  'position'=>'bottom',
                  'target'=>'.button-new',
                  'type'=>2,
                  'intertype'=>1,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_TITLE_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_TITLE_DESC',
                  'position'=>'bottom',
                  'target'=>'#jform_title',
                  'type'=>2,
                  'intertype'=>2,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_DATE_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_DATE_DESC',
                  'position'=>'bottom',
                  'target'=>'#jform_motion_date',
                  'type'=>2,
                  'intertype'=>2,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_CAT_ID_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_CAT_ID_DESC',
                  'position'=>'top',
                  'target'=>'#jform_cat_id',
                  'type'=>2,
                  'intertype'=>2,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_MOTION_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_MOTION_DESC',
                  'position'=>'bottom',
                  'target'=>'#jform_motion,#jform_motion_ifr',
                  'type'=>2,
                  'intertype'=>3,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_TAB_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_TAB_DESC',
                  'position'=>'bottom',
                  'target'=>'button[aria-controls=general]',
                  'type'=>2,
                  'intertype'=>4,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_USERGP_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_USERGP_DESC',
                  'position'=>'top',
                  'target'=>'joomla-field-fancy-select .choices',
                  'type'=>2,
                  'intertype'=>2,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_EXPIRY_DATE_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_EXPIRY_DATE_DESC',
                  'position'=>'top',
                  'target'=>'#jform_expiry_date',
                  'type'=>2,
                  'intertype'=>2,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_REQSEC_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_REQSEC_DESC',
                  'position'=>'bottom',
                  'target'=>'#jform_reqsec',
                  'type'=>2,
                  'intertype'=>2,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_TAB_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_TAB_DESC',
                  'position'=>'bottom',
                  'target'=>'button[aria-controls=extrainfo]',
                  'type'=>2,
                  'intertype'=>4,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_FIELD_ID_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_MOTION_FIELD_ID_DESC',
                  'position'=>'top',
                  'target'=>'#jform_field_id',
                  'type'=>2,
                  'intertype'=>2,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_SAVECLOSE_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_SAVECLOSE_DESC',
                  'position'=>'bottom',
                  'target'=>'#toolbar-save button.button-save',
                  'type'=>2,
                  'intertype'=>1,
                  'url'=>$returnURL
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONGRATS_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONGRATS_DESC',
                  'position'=>'bottom',
                  'target'=>'',
                  'type'=>0,
                  'intertype'=>1,
                  'url'=>$returnURL
                  );
        }

        if ($ref == 'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_LBL') {

            $returnURL = 'administrator/index.php?option=com_'.$this->compName.'&view='.$this->mainView;
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_SETTINGS_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_SETTINGS_DESC',
                  'position'=>'bottom',
                  'target'=>'div.cpanel-modules.cpanel-'.$this->compName.' ul.list-group li.list-group-item a.flex-grow-1',
                  'type'=>2,
                  'intertype'=>1,
                  'url'=>'administrator/index.php?option=com_config&view=component&component=com_'.$this->compName.'&path=&return='
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_TAB_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_TAB_DESC',
                  'position'=>'bottom',
                  'target'=>'button[aria-controls=integration]',
                  'type'=>2,
                  'intertype'=>4,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_SENDEMAIL_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_SENDEMAIL_DESC',
                  'position'=>'bottom',
                  'target'=>'#jform_send_email',
                  'type'=>2,
                  'intertype'=>2,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_TMPLEMAIL_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_TMPLEMAIL_DESC',
                  'position'=>'bottom',
                  'target'=>'#jform_tmpl_email',
                  'type'=>2,
                  'intertype'=>2,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_TAB_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_TAB_DESC',
                  'position'=>'bottom',
                  'target'=>'button[aria-controls=permissions]',
                  'type'=>2,
                  'intertype'=>4,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_CREATE_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_CREATE_DESC',
                  'position'=>'bottom',
                  'target'=>'',
                  'type'=>2,
                  'intertype'=>2,
                  'url'=>''
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_SAVECLOSE_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_SAVECLOSE_DESC',
                  'position'=>'bottom',
                  'target'=>'#toolbar #toolbar-save button.button-save',
                  'type'=>2,
                  'intertype'=>1,
                  'url'=>$returnURL
                  );
            $recs[] = array(
                  'title'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONGRATS_LBL',
                  'desc'=>'COM_'.STRTOUPPER($this->compName).'_GUIDEDTOUR_CONFIG_CONGRATS_DESC',
                  'position'=>'bottom',
                  'target'=>'',
                  'type'=>0,
                  'intertype'=>1,
                  'url'=>$returnURL
                  );
        }

		return $recs;
	}

	/**
	 * *********************  Dashborad stuff  *******************************
	 */
	/**
	 * Check if a Dashboard module entry exists
	 * @param   string $component Component name
	 * @return boolean or object
	 */
	public function checkDashboard($component = 0)
	{
        $result = false;
		if ($component) {
			$db = Factory::getContainer()->get('DatabaseDriver');
	        $db->setQuery(' SELECT * FROM #__modules WHERE position = '.$db->Quote('cpanel-'.$component) );
		    try {
		        $result = $db->loadObjectList();
		    } catch (RuntimeException $e) {
		        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
		    }
	    }

		return $result;
	}

	/**
	 * Removes the dashboard menu module
	 * @param int $id The dashboard module id reference
	 * @return  void
	 */
	public function removeDashboardMenu($id)
	{
		$model  = Factory::getApplication()->bootComponent('com_modules')->getMVCFactory()->createModel('Module', 'Administrator', ['ignore_request' => true]);
        $table = $model->getTable();
        $table->load($id);

		if (!$table->delete())
		{
			Factory::getApplication()->enqueueMessage(Text::sprintf('COM_'.STRTOUPPER($this->compName).'_REMOVE_DASHBOARD_FAIL', $model->getError()));
		}
	}

	/**
	 * *********************  MailTemplate stuff  *******************************
	 */
	/**
	 * Install MailTemplate entrys
	 * @return boolean or object
	 */
	public function installMailTemplates()
	{
        foreach ($this->mailTmplSuffixs as $tmpl) {
            $template_id = 'com_'.$this->compName.'.'.$tmpl;
            $tmplExists = $this->checkMailTemplates($template_id);
            if (!$tmplExists) {
                $this->addMailTemplate($tmpl, $this->mailTags);
            } else {
                $this->updateMailTemplates($tmplExists);
            }
        }

		return true;
	}

	/**
	 * Update template records if tags changed
	 * @param   object $tmpl
	 */
	public function updateMailTemplates($tmpl)
	{
        // check tags
        $params = json_decode($tmpl->params);

        if (count($params->tags) != count($this->mailTags)) {
            $params->tags = $this->mailTags;
            $tmpl->params = json_encode($params);
            $result = Factory::getContainer()->get('DatabaseDriver')->updateObject('#__mail_templates', $tmpl, 'template_id');
            Factory::getApplication()->enqueueMessage(Text::sprintf('COM_'.STRTOUPPER($this->compName).'_MAILTMPL_UPDATED', $tmpl->template_id), 'success');
        }

	}

	/**
	 * Check if a MailTemplate entry exists
	 * @param   string $template_id (component + ext)
	 * @return boolean or object
	 */
	public function checkMailTemplates($template_id = 0)
	{
        $result = false;
		if ($template_id) {
			$db = Factory::getContainer()->get('DatabaseDriver');
	        $db->setQuery(' SELECT * FROM #__mail_templates WHERE template_id = '.$db->Quote($template_id) );
		    try {
		        $result = $db->loadObject();
		    } catch (RuntimeException $e) {
		        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
		    }
	    }

		return $result;
	}

	/**
	 * Removes the mailTemplate records
	 * @param   string $template_id (component + ext)
	 * @return  void
	 */
	public function removeMailTemplate($template_id)
	{
		$this->deleteData('#__mail_templates', 'template_id', $template_id, '=');
		Factory::getApplication()->enqueueMessage(Text::_('COM_'.STRTOUPPER($this->compName ?? '').'_EMAILTMPL_REMOVE_SUCCESS', 'notice'));
	}

	/**
	 * Create a new Mail Template
	 * @param   string $template_id (just the ext)
	 * @param   array $tags
	 * @return  void
	 */
	public function addMailTemplate($template_id, $tags)
	{
        $result = MailTemplate::createTemplate(
        	'com_'.$this->compName.'.'.$template_id,
        	'COM_'.STRTOUPPER($this->compName).'_EMAILTMPL_'.STRTOUPPER($template_id).'_SUBJECT',
        	'COM_'.STRTOUPPER($this->compName).'_EMAILTMPL_'.STRTOUPPER($template_id).'_BODY',
        	$tags,
        	'COM_'.STRTOUPPER($this->compName).'_EMAILTMPL_'.STRTOUPPER($template_id).'_HTMLBODY'
        );

		Factory::getApplication()->enqueueMessage('Email Template Record Created . . . '.$template_id);

	}

	/**
	 * *********************  Data and Files stuff  *******************************
	 */
	/**
	 * Update data
	 * @param string $table Table name to be updated
	 * @param string $fieldname field name to be updated
	 * @param string $newvalue value to set the above fieldname
	 * @param string $wherefield field name of the where clause to test for
	 * @param string $oldvalue value of the field name in the where clause
	 * @param string $opatr operator for the test in the where clause
	 * @return  void
	 */
	public function updateData($table = '#__user_profiles', $wherefield = 'profile_key', $fieldname = 'profile_value', $oldvalue = 'Life', $newvalue = 'Life Member', $opatr = '=')
	{
		$db    = Factory::getContainer()->get('DatabaseDriver');
		$query = $db->getQuery(true);
		$db->setQuery('UPDATE '.$table.' SET '.$fieldname.' = '.$newvalue.' WHERE '.$wherefield.' '.$opatr.' '.$db->quote($oldvalue));
		$db->execute();
		Factory::getApplication()->enqueueMessage('Records updated . . . '.$table);

	}

	/**
	 * Delete old data
	 */
	public function deleteData($table = '#__extensions', $field = 'element', $value = '%com_weblinks%', $opatr = 'LIKE')
	{
		$db    = Factory::getContainer()->get('DatabaseDriver');
		$query = $db->getQuery(true);
		$db->setQuery('DELETE from '.$table.' WHERE '.$field.' '.$opatr.' '.$db->quote($value));
		$db->execute();
		Factory::getApplication()->enqueueMessage('Records removed from . . . '.$table);

	}

	/**
	 * Drop old tables
	 */
	public function dropTable($table = null)
	{
		$db    = Factory::getContainer()->get('DatabaseDriver');
		$query = $db->getQuery(true);
		$db->setQuery('DROP TABLE IF EXISTS '.$table);
		$db->execute();
		Factory::getApplication()->enqueueMessage('Table dropped . . . '.$table);

	}

	/**
	 * Create data
	 */
	public function createData()
	{
		$rec = new \stdClass();
		$rec->id = 0;
		$rec->ordering = 0;
		$rec->state = 1;
		$rec->checked_out = 0;
		$rec->checked_out_time = NULL;
		$rec->created_by = 0;
		$rec->created_date = '2020-07-23 12:32:00';

        foreach ($days AS $ddate) {
            $rec->diary_date = $ddate;
            $result = Factory::getContainer()->get('DatabaseDriver')->insertObject('#__garafsqn_diarys', $rec);
        }

		Factory::getApplication()->enqueueMessage('Records created . . . ');

	}

	/**
	 * Delete unwanted file
	 */
	public function deleteFiles($path = null, $file = null, $ext = null)
	{
        $path_to_file = $path . $file . $ext;
		if (is_file($path_to_file)) {
			File::delete($path_to_file);
			Factory::getApplication()->enqueueMessage('Deleted File - '.$path_to_file);
		}
	}

	/**
	 * Delete unwanted folder
	 */
	public function deleteFolder($path = null)
	{
		if (is_dir($path)) {
			Folder::delete($path);
			Factory::getApplication()->enqueueMessage('Deleted Folder - '.$path);
		}
	}

	/**
	 * Create a folder
	 */
	public function createFolder($parent = null, $folder = null)
	{
        $path = Path::clean( JPATH_SITE . '/' . $parent . '/' . $folder );

		if (!is_dir($path) && !is_file($path)) {
            Folder::create($path);
			Factory::getApplication()->enqueueMessage('Folder Created - '.$path);
        } else {
			Factory::getApplication()->enqueueMessage('Folder Already Exists - '.$path);
		}
	}

	/**
	 * Get the version of the component
	 * @return version element of manifest
	 */
	public static function getComponentVersion($component)
	{
		$componentTest = ComponentHelper::getComponent($component, true);
        if (empty($componentTest->enabled)) {
            return $this->version;
        } else {
            $componentXML = Installer::parseXMLInstallFile(Path::clean(JPATH_ADMINISTRATOR . '/components/com_'.$component.'/'.$component.'.xml'));
            return $componentXML['version'];
        }
	}

	/**
	 * ***************  Check for old data structure where columns need to be added  ***************
	 */

	/**
	 * Check for a columns in table
	 */
	private function checkColumns()
	{
		$table = '#__gausers_profile_audit';
		$column = 'post_update';
		$after = 'user_id';
		$type = 'text';

        $colExists = $this->checkColumnExists($table, $column);
		if (empty($colExists)) {
            Factory::getApplication()->enqueueMessage(Text::_('Column '.$column.' NOT Exists'), 'warning');
            $this->createColumn($table, $column, $after, $type);
        }

		$column = 'pre_update';
        $colExists = $this->checkColumnExists($table, $column);
		if (empty($colExists)) {
            Factory::getApplication()->enqueueMessage(Text::_('Column '.$column.' NOT Exists'), 'warning');
            $this->createColumn($table, $column, $after, $type);
        }

	}

	/**
	 * Check if a Database field exists
	 * @param   string $table
	 * @param   string $field
	 * @param   string $type
	 * @return boolean
	 */
	public function checkDBFields($table, $field, $ftype)
	{
		$db = Factory::getContainer()->get('DatabaseDriver');
	    $db->setQuery(' SELECT count(*) FROM information_schema.columns WHERE table_schema = '.$db->Quote($this->dbName).' AND table_name ='.$db->Quote($table).' AND column_name = '.$db->Quote($field) );
		try {
		    $return = $db->loadResult();
		} catch (RuntimeException $e) {
		    $return = false;
            Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
		}
		return $return;
	}

	/**
	 * Check for a column in table
	 * @param   table name
	 * @param   column name
	 * @return  boolean
	 */
	private function checkColumnExists($table, $column)
	{
		$db    = Factory::getContainer()->get('DatabaseDriver');
		$db->setQuery('SHOW COLUMNS FROM '.$db->quotename($table) . ' LIKE '.$db->quote($column));
		try {
		    return $db->loadObject();
		} catch (RuntimeException $e) {
		    Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
		    return false;
		}
	}

	/**
	 * Check for a column in table
	 * @param   table name
	 * @param   column name
	 * @return  boolean
	 */
	private function createColumn($table, $column, $after, $type)
	{
		$db    = Factory::getContainer()->get('DatabaseDriver');
		$db->setQuery('ALTER TABLE '.$db->quotename($table) . ' ADD '.$db->quotename($column).' '.$type.' NULL AFTER '.$db->quotename($after));
		try {
		    $db->execute();
		    Factory::getApplication()->enqueueMessage('Column '. $column . ' created', 'warning');
		} catch (RuntimeException $e) {
		    Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
		}
	}

	/**
	 * *********************  All the plugin installation stuff  *******************************
	 */

	/**
	 * Installs plugins for this component
	 * @param   mixed $parent Object who called the install/update method
	 * @return void
	 */
	private function installPlugins($parent)
	{
		$installation_folder = $parent->getParent()->getPath('source');
		$app                 = Factory::getApplication();

		/* @var $plugins SimpleXMLElement */
		if (method_exists($parent, 'getManifest'))
		{
			$plugins = $parent->getManifest()->plugins;
		}
		else
		{
			$plugins = $parent->get('manifest')->plugins;
		}

		if (count($plugins->children()))
		{
			$db = Factory::getContainer()->get('DatabaseDriver');
			$query = $db->getQuery(true);

			foreach ($plugins->children() as $plugin)
			{
				$pluginName  = (string) $plugin['plugin'];
				$pluginGroup = (string) $plugin['group'];
				$path        = $installation_folder . '/plugins/' . $pluginGroup . '/' . $pluginName;
				$installer   = new Installer;

				if (!$this->isAlreadyInstalled('plugin', $pluginName, $pluginGroup))
				{
					$result = $installer->install($path);
				}
				else
				{
					$result = $installer->update($path);
				}

				if ($result)
				{
					$app->enqueueMessage('Plugin ' . $pluginName . ' was installed successfully');
				}
				else
				{
					$app->enqueueMessage('There was an issue installing the plugin ' . $pluginName,
						'error');
				}

				$query
					->clear()
					->update('#__extensions')
					->set('enabled = 1')
					->where(
						array(
							'type LIKE ' . $db->quote('plugin'),
							'element LIKE ' . $db->quote($pluginName),
							'folder LIKE ' . $db->quote($pluginGroup)
						)
					);
				$db->setQuery($query);
				$db->execute();
			}
		}
	}

	/**
	 * Uninstalls plugins
	 * @param   mixed $parent Object who called the uninstall method
	 * @return void
	 */
	private function uninstallPlugins($parent)
	{
		$app     = Factory::getApplication();

		if (method_exists($parent, 'getManifest'))
		{
			$plugins = $parent->getManifest()->plugins;
		}
		else
		{
			$plugins = $parent->get('manifest')->plugins;
		}

		if (count($plugins->children()))
		{
			$db = Factory::getContainer()->get('DatabaseDriver');
			$query = $db->getQuery(true);

			foreach ($plugins->children() as $plugin)
			{
				$pluginName  = (string) $plugin['plugin'];
				$pluginGroup = (string) $plugin['group'];
				$query
					->clear()
					->select('extension_id')
					->from('#__extensions')
					->where(
						array(
							'type LIKE ' . $db->quote('plugin'),
							'element LIKE ' . $db->quote($pluginName),
							'folder LIKE ' . $db->quote($pluginGroup)
						)
					);
				$db->setQuery($query);
				$extension = $db->loadResult();

				if (!empty($extension))
				{
					$installer = new Installer;
					$result    = $installer->uninstall('plugin', $extension);

					if ($result)
					{
						$app->enqueueMessage('Plugin ' . $pluginName . ' was uninstalled successfully');
					}
					else
					{
						$app->enqueueMessage('There was an issue uninstalling the plugin ' . $pluginName,
							'error');
					}
				}
			}
		}
	}

	/**
	 * *********************  All the module installation stuff  *******************************
	 */

	/**
	 * Installs modules for this component
	 * @param   mixed $parent Object who called the install/update method
	 * @return void
	 */
	private function installModules($parent)
	{
		$installation_folder = $parent->getParent()->getPath('source');
		$app                 = Factory::getApplication();

		if (method_exists($parent, 'getManifest'))
		{
			$modules = $parent->getManifest()->modules;
		}
		else
		{
			$modules = $parent->get('manifest')->modules;
		}

		if (!empty($modules))
		{

			if (count($modules->children()))
			{
				foreach ($modules->children() as $module)
				{
					$moduleName = (string) $module['module'];
					$path       = $installation_folder . '/modules/' . $moduleName;
					$installer  = new Installer;

					if (!$this->isAlreadyInstalled('module', $moduleName))
					{
						$result = $installer->install($path);
					}
					else
					{
						$result = $installer->update($path);
					}

					if ($result)
					{
						$app->enqueueMessage('Module ' . $moduleName . ' was installed successfully');
					}
					else
					{
						$app->enqueueMessage('There was an issue installing the module ' . $moduleName,
							'error');
					}
				}
			}
		}
	}

	/**
	 * Uninstalls modules
	 * @param   mixed $parent Object who called the uninstall method
	 * @return void
	 */
	private function uninstallModules($parent)
	{
		$app = Factory::getApplication();

		if (method_exists($parent, 'getManifest'))
		{
			$modules = $parent->getManifest()->modules;
		}
		else
		{
			$modules = $parent->get('manifest')->modules;
		}

		if (!empty($modules))
		{

			if (count($modules->children()))
			{
				$db = Factory::getContainer()->get('DatabaseDriver');
				$query = $db->getQuery(true);

				foreach ($modules->children() as $plugin)
				{
					$moduleName = (string) $plugin['module'];
					$query
						->clear()
						->select('extension_id')
						->from('#__extensions')
						->where(
							array(
								'type LIKE ' . $db->quote('module'),
								'element LIKE ' . $db->quote($moduleName)
							)
						);
					$db->setQuery($query);
					$extension = $db->loadResult();

					if (!empty($extension))
					{
						$installer = new Installer;
						$result    = $installer->uninstall('module', $extension);

						if ($result)
						{
							$app->enqueueMessage('Module ' . $moduleName . ' was uninstalled successfully');
						}
						else
						{
							$app->enqueueMessage('There was an issue uninstalling the module ' . $moduleName,
								'error');
						}
					}
				}
			}
		}
	}

	/**
	 * Check if an extension is already installed in the system
	 * @param   string $type   Extension type
	 * @param   string $name   Extension name
	 * @param   mixed  $folder Extension folder(for plugins)
	 * @return boolean
	 */
	private function isAlreadyInstalled($type, $name, $folder = null)
	{
		$result = false;

		switch ($type)
		{
			case 'plugin':
				$result = file_exists(JPATH_PLUGINS . '/' . $folder . '/' . $name);
				break;
			case 'module':
				$result = file_exists(JPATH_SITE . '/modules/' . $name);
				break;
		}

		return $result;
	}

}
