<?php
/**
 * @version    1.0.2
 * @package    com_gadecisions
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Layout\LayoutHelper;
use \GlennArkell\Component\Gadecisions\Administrator\Helper\GadecisionsHelper;

// load any assets required
$this->document->getWebAssetManager()
    ->usePreset('com_gadecisions.gadecisionspreset')
    ->useScript('keepalive')
	->useScript('form.validate');

// Load admin language file
$lang = Factory::getLanguage();
$lang->load('com_gadecisions', JPATH_ADMINISTRATOR, 'en-GB', true);

$user    = GadecisionsHelper::getSpecificUser();
$canEdit = GadecisionsHelper::canUserEdit($user, $this->item);
$canAdmin = $user->authorise('core.admin', 'com_gadecisions');

?>

<div class="motion-edit front-end-edit">
	<?php if (!$canAdmin) : ?>
		<h3>
			<?php Factory::getApplication()->enqueueMessage(Text::_('COM_GADECISIONS_ERROR_MESSAGE_NOT_AUTHORISED'), 'danger'); ?>
		</h3>
	<?php else : ?>
		<?php if (!empty($this->item->id)): ?>
			<h1><?php echo Text::sprintf('COM_GADECISIONS_EDIT_ITEM_TITLE', $this->item->id); ?></h1>
		<?php else: ?>
			<h1><?php echo Text::_('COM_GADECISIONS_ADD_ITEM_TITLE'); ?></h1>
		<?php endif; ?>

		<form id="form-motion"
			  action="<?php echo Route::_('index.php?option=com_gadecisions&task=motionform.save'); ?>"
			  method="post" class="form-validate form-horizontal" enctype="multipart/form-data">

			<?php echo HTMLHelper::_('uitab.startTabSet', 'myTab', array('active' => 'motion')); ?>
			<?php echo HTMLHelper::_('uitab.addTab', 'myTab', 'motion', Text::_('COM_GADECISIONS_TITLE_MOTION', true)); ?>
		
				<div class="row-fluid">
					<div class="span10 form-horizontal">
						<fieldset name="motion" class="adminform">
							<legend><?php echo Text::_('COM_GADECISIONS_TITLE_MOTION'); ?></legend>
							<?php echo $this->form->renderFieldset('motion'); ?>
						</fieldset>
					</div>
				</div>

			<?php echo HTMLHelper::_('uitab.endTab'); ?>

			<?php echo HTMLHelper::_('uitab.addTab', 'myTab', 'general', Text::_('COM_GADECISIONS_TITLE_GENERAL', true)); ?>
				<div class="row-fluid">
					<div class="span10 form-horizontal">
						<fieldset name="general" class="adminform">
							<?php echo $this->form->renderFieldset('general'); ?>
						</fieldset>
					</div>
				</div>
			<?php echo HTMLHelper::_('uitab.endTab'); ?>
		
			<?php echo HTMLHelper::_('uitab.addTab', 'myTab', 'extrainfo', Text::_('COM_GADECISIONS_XTRAINFO', true)); ?>
				<div class="row-fluid">
					<div class="span10 form-horizontal">
						<fieldset name="extrainfo" class="adminform">
							<?php echo $this->form->renderFieldset('extrainfo'); ?>
						</fieldset>
					</div>
				</div>
			<?php echo HTMLHelper::_('uitab.endTab'); ?>
		
			<?php echo HTMLHelper::_('uitab.addTab', 'myTab', 'sysinfo', Text::_('COM_GADECISIONS_SYSINFO', true)); ?>
				<div class="row-fluid">
					<div class="span10 form-horizontal">
						<fieldset name="sysinfo" class="adminform">
							<?php echo $this->form->renderFieldset('sysinfo'); ?>
						</fieldset>
					</div>
				</div>
			<?php echo HTMLHelper::_('uitab.endTab'); ?>
	
			<?php echo HTMLHelper::_('uitab.endTabSet'); ?>

			<div class="control-group">
				<div class="controls">

					<?php if ($this->canSave): ?>
						<button type="submit" class="validate btn btn-primary">
							<span class="fas fa-check" aria-hidden="true"></span>
							<?php echo Text::_('JSUBMIT'); ?>
						</button>
					<?php endif; ?>
					<a class="btn btn-danger"
					   href="<?php echo Route::_('index.php?option=com_gadecisions&task=motionform.cancel'); ?>"
					   title="<?php echo Text::_('JCANCEL'); ?>">
					   <span class="fas fa-times" aria-hidden="true"></span>
						<?php echo Text::_('JCANCEL'); ?>
					</a>
				</div>
			</div>

			<input type="hidden" name="option" value="com_gadecisions"/>
			<input type="hidden" name="task" value="motionform.save"/>
			<?php echo HTMLHelper::_('form.token'); ?>
		</form>
	<?php endif; ?>
</div>
